from csv import reader as CSVReader
import numpy as np

NTEAMS = 11
NSTUDENTMAX = 150

def feat_trans(rank):
    y, x = np.meshgrid(rank, rank, index='ij')
    return np.sign(y-x)


with open("ranks.csv") as fpRank,\
     open("corr.txt", "w") as fpCorr:

    winloseTable = np.zeros((NTEAMS, NTEAMS))
    rows = list(CSVReader(fpRank))

    # read the datum
    names = []
    weights = np.empty(NSTUDENTMAX)
    ranks = np.empty((NSTUDENTMAX, NTEAMS))
    validIdx = np.zeros(NSTUDENTMAX, dtype=np.bool_)
    for i, row in enumerate(rows):
        names.append(row[0])
        weights[i] = float(row[1])
        rankList = [float(c) for c in row[2:] if c]
        if len(rankList) == NTEAMS:
            ranks[i] = rankList
            validIdx[i] = True
    ranks = ranks[:i+1]
    weights = weights[:i+1]
    validIdx = validIdx[:i+1]

    # feature transform & voting
    for i, v in enumerate(validIdx):
        if v:
            winloseTable += feat_trans(ranks[i]) * weights[i]

    votingRes = winloseTable.sum(1)
    finalOrder = np.argsort(votingRes)[::-1]
    finalRank = np.arange(NTEAMS)
    finalRank[finalOrder] = np.arange(NTEAMS)

    # announce final result
    for i, team in enumerate(finalOrder):
        print "Rank {:2d}: team {:2d} get {:>5.0f} votes".format(i+1, int(team+1), votingRes[team])

    # interesting: whose opnion is similar to average opnion
    avgFeat = feat_trans(finalRank)
    for i, name in enumerate(names):
        if validIdx[i]:
            feat = feat_trans(ranks[i])
            innProd = (feat * avgFeat).sum()
            innProdNrm = innProd / (NTEAMS*(NTEAMS-1)) # -1 ~ 1
            score = (innProdNrm + 1) * 50 # 0 ~ 100
            fpCorr.write("{}: {}\n".format(name, int(score)))
        else:
            fpCorr.write(name + ": N/A\n")

    # randomly write vote result
    ranks = ranks[validIdx]
    np.random.shuffle(ranks)
    np.savetxt("ranks_shuffle.txt", ranks, fmt="%2.1f", delimiter=', ')
